/*
 * This file was generated automatically by ExtUtils::ParseXS version 3.63 from the
 * contents of CairoGObject.xs. Do not edit this file, edit CairoGObject.xs instead.
 *
 *    ANY CHANGES MADE HERE WILL BE LOST!
 *
 */

#line 1 "CairoGObject.xs"
/*
 * Copyright (c) 2011 by the cairo perl team (see the file README)
 *
 * Licensed under the LGPL, see LICENSE file for more information.
 *
 */

#include <cairo-perl.h>
#include <gperl.h>

#include <cairo-gobject.h>

/* ------------------------------------------------------------------------- */

static SV *
context_wrap (GType gtype, const char * package, gpointer boxed, gboolean own)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return own ?
		newSVCairo_noinc ((cairo_t *) boxed) :
		newSVCairo ((cairo_t *) boxed);
}

gpointer
context_unwrap (GType gtype, const char * package, SV * sv)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return SvCairo_ornull (sv);
}

GPerlBoxedWrapperClass context_wrapper_class = {
	context_wrap,
	context_unwrap,
	NULL,
};

/* ------------------------------------------------------------------------- */

static SV *
pattern_wrap (GType gtype, const char * package, gpointer boxed, gboolean own)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return own ?
		newSVCairoPattern_noinc ((cairo_pattern_t *) boxed) :
		newSVCairoPattern ((cairo_pattern_t *) boxed);
}

gpointer
pattern_unwrap (GType gtype, const char * package, SV * sv)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return SvCairoPattern_ornull (sv);
}

GPerlBoxedWrapperClass pattern_wrapper_class = {
	pattern_wrap,
	pattern_unwrap,
	NULL,
};

/* ------------------------------------------------------------------------- */

static SV *
surface_wrap (GType gtype, const char * package, gpointer boxed, gboolean own)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return own ?
		newSVCairoSurface_noinc ((cairo_surface_t *) boxed) :
		newSVCairoSurface ((cairo_surface_t *) boxed);
}

gpointer
surface_unwrap (GType gtype, const char * package, SV * sv)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return SvCairoSurface_ornull (sv);
}

GPerlBoxedWrapperClass surface_wrapper_class = {
	surface_wrap,
	surface_unwrap,
	NULL,
};

/* ------------------------------------------------------------------------- */

static SV *
rectangle_wrap (GType gtype, const char * package, gpointer boxed, gboolean own)
{
	SV * sv;
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	sv = newSVCairoRectangle ((cairo_rectangle_t *) boxed);
	if (own) {
		/* FIXME: What if some other allocator was used? */
		g_free (boxed);
	}
	return sv;
}

gpointer
rectangle_unwrap (GType gtype, const char * package, SV * sv)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return SvCairoRectangle_ornull (sv);
}

GPerlBoxedWrapperClass rectangle_wrapper_class = {
	rectangle_wrap,
	rectangle_unwrap,
	NULL,
};

/* ------------------------------------------------------------------------- */

static SV *
rectangle_int_wrap (GType gtype, const char * package, gpointer boxed, gboolean own)
{
	SV * sv;
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	sv = newSVCairoRectangleInt ((cairo_rectangle_int_t *) boxed);
	if (own) {
		/* FIXME: What if some other allocator was used? */
		g_free (boxed);
	}
	return sv;
}

gpointer
rectangle_int_unwrap (GType gtype, const char * package, SV * sv)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return SvCairoRectangleInt_ornull (sv);
}

GPerlBoxedWrapperClass rectangle_int_wrapper_class = {
	rectangle_int_wrap,
	rectangle_int_unwrap,
	NULL,
};

/* ------------------------------------------------------------------------- */

static SV *
region_wrap (GType gtype, const char * package, gpointer boxed, gboolean own)
{
	SV * sv;
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	sv = newSVCairoRegion ((cairo_region_t *) boxed);
	if (own) {
		/* FIXME: What if some other allocator was used? */
		g_free (boxed);
	}
	return sv;
}

gpointer
region_unwrap (GType gtype, const char * package, SV * sv)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return SvCairoRegion_ornull (sv);
}

GPerlBoxedWrapperClass region_wrapper_class = {
	region_wrap,
	region_unwrap,
	NULL,
};

/* ------------------------------------------------------------------------- */

static SV *
scaled_font_wrap (GType gtype, const char * package, gpointer boxed, gboolean own)
{
	SV * sv;
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	sv = newSVCairoScaledFont ((cairo_scaled_font_t *) boxed);
	if (own) {
		/* FIXME: What if some other allocator was used? */
		g_free (boxed);
	}
	return sv;
}

gpointer
scaled_font_unwrap (GType gtype, const char * package, SV * sv)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return SvCairoScaledFont_ornull (sv);
}

GPerlBoxedWrapperClass scaled_font_wrapper_class = {
	scaled_font_wrap,
	scaled_font_unwrap,
	NULL,
};

/* ------------------------------------------------------------------------- */

static SV *
font_face_wrap (GType gtype, const char * package, gpointer boxed, gboolean own)
{
	SV * sv;
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	sv = newSVCairoFontFace ((cairo_font_face_t *) boxed);
	if (own) {
		/* FIXME: What if some other allocator was used? */
		g_free (boxed);
	}
	return sv;
}

gpointer
font_face_unwrap (GType gtype, const char * package, SV * sv)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return SvCairoFontFace_ornull (sv);
}

GPerlBoxedWrapperClass font_face_wrapper_class = {
	font_face_wrap,
	font_face_unwrap,
	NULL,
};

/* ------------------------------------------------------------------------- */

static SV *
font_options_wrap (GType gtype, const char * package, gpointer boxed, gboolean own)
{
	SV * sv;
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	sv = newSVCairoFontOptions ((cairo_font_options_t *) boxed);
	if (own) {
		/* FIXME: What if some other allocator was used? */
		g_free (boxed);
	}
	return sv;
}

gpointer
font_options_unwrap (GType gtype, const char * package, SV * sv)
{
	PERL_UNUSED_VAR (gtype);
	PERL_UNUSED_VAR (package);
	return SvCairoFontOptions_ornull (sv);
}

GPerlBoxedWrapperClass font_options_wrapper_class = {
	font_options_wrap,
	font_options_unwrap,
	NULL,
};

/* ------------------------------------------------------------------------- */

#line 283 "CairoGObject.c"
#ifndef PERL_UNUSED_VAR
#  define PERL_UNUSED_VAR(var) if (0) var = var
#endif

#ifndef dVAR
#  define dVAR		dNOOP
#endif


/* This stuff is not part of the API! You have been warned. */
#ifndef PERL_VERSION_DECIMAL
#  define PERL_VERSION_DECIMAL(r,v,s) (r*1000000 + v*1000 + s)
#endif
#ifndef PERL_DECIMAL_VERSION
#  define PERL_DECIMAL_VERSION \
	  PERL_VERSION_DECIMAL(PERL_REVISION,PERL_VERSION,PERL_SUBVERSION)
#endif
#ifndef PERL_VERSION_GE
#  define PERL_VERSION_GE(r,v,s) \
	  (PERL_DECIMAL_VERSION >= PERL_VERSION_DECIMAL(r,v,s))
#endif
#ifndef PERL_VERSION_LE
#  define PERL_VERSION_LE(r,v,s) \
	  (PERL_DECIMAL_VERSION <= PERL_VERSION_DECIMAL(r,v,s))
#endif

/* XS_INTERNAL is the explicit static-linkage variant of the default
 * XS macro.
 *
 * XS_EXTERNAL is the same as XS_INTERNAL except it does not include
 * "static", ie. it exports XSUB symbols. You probably don't want that
 * for anything but the BOOT XSUB.
 *
 * See XSUB.h in core!
 */


/* TODO: This might be compatible further back than 5.10.0. */
#if PERL_VERSION_GE(5, 10, 0) && PERL_VERSION_LE(5, 15, 1)
#  undef XS_EXTERNAL
#  undef XS_INTERNAL
#  if defined(__CYGWIN__) && defined(USE_DYNAMIC_LOADING)
#    define XS_EXTERNAL(name) __declspec(dllexport) XSPROTO(name)
#    define XS_INTERNAL(name) static XSPROTO(name)
#  endif
#  if defined(__SYMBIAN32__)
#    define XS_EXTERNAL(name) EXPORT_C XSPROTO(name)
#    define XS_INTERNAL(name) EXPORT_C static XSPROTO(name)
#  endif
#  ifndef XS_EXTERNAL
#    if defined(HASATTRIBUTE_UNUSED) && !defined(__cplusplus)
#      define XS_EXTERNAL(name) void name(pTHX_ CV* cv __attribute__unused__)
#      define XS_INTERNAL(name) static void name(pTHX_ CV* cv __attribute__unused__)
#    else
#      ifdef __cplusplus
#        define XS_EXTERNAL(name) extern "C" XSPROTO(name)
#        define XS_INTERNAL(name) static XSPROTO(name)
#      else
#        define XS_EXTERNAL(name) XSPROTO(name)
#        define XS_INTERNAL(name) static XSPROTO(name)
#      endif
#    endif
#  endif
#endif

/* perl >= 5.10.0 && perl <= 5.15.1 */


/* The XS_EXTERNAL macro is used for functions that must not be static
 * like the boot XSUB of a module. If perl didn't have an XS_EXTERNAL
 * macro defined, the best we can do is assume XS is the same.
 * Dito for XS_INTERNAL.
 */
#ifndef XS_EXTERNAL
#  define XS_EXTERNAL(name) XS(name)
#endif
#ifndef XS_INTERNAL
#  define XS_INTERNAL(name) XS(name)
#endif

/* Now, finally, after all this mess, we want an ExtUtils::ParseXS
 * internal macro that we're free to redefine for varying linkage due
 * to the EXPORT_XSUB_SYMBOLS XS keyword. This is internal, use
 * XS_EXTERNAL(name) or XS_INTERNAL(name) in your code if you need to!
 */

#undef XS_EUPXS
#if defined(PERL_EUPXS_ALWAYS_EXPORT)
#  define XS_EUPXS(name) XS_EXTERNAL(name)
#else
   /* default to internal */
#  define XS_EUPXS(name) XS_INTERNAL(name)
#endif

#ifndef PERL_ARGS_ASSERT_CROAK_XS_USAGE
#define PERL_ARGS_ASSERT_CROAK_XS_USAGE assert(cv); assert(params)

/* prototype to pass -Wmissing-prototypes */
static void
S_croak_xs_usage(const CV *const cv, const char *const params);

static void
S_croak_xs_usage(const CV *const cv, const char *const params)
{
    const GV *const gv = CvGV(cv);

    PERL_ARGS_ASSERT_CROAK_XS_USAGE;

    if (gv) {
        const char *const gvname = GvNAME(gv);
        const HV *const stash = GvSTASH(gv);
        const char *const hvname = stash ? HvNAME(stash) : NULL;

        if (hvname)
	    Perl_croak_nocontext("Usage: %s::%s(%s)", hvname, gvname, params);
        else
	    Perl_croak_nocontext("Usage: %s(%s)", gvname, params);
    } else {
        /* Pants. I don't think that it should be possible to get here. */
	Perl_croak_nocontext("Usage: CODE(0x%" UVxf ")(%s)", PTR2UV(cv), params);
    }
}
#undef  PERL_ARGS_ASSERT_CROAK_XS_USAGE

#define croak_xs_usage        S_croak_xs_usage

#endif

/* NOTE: the prototype of newXSproto() is different in versions of perls,
 * so we define a portable version of newXSproto()
 */
#ifdef newXS_flags
#define newXSproto_portable(name, c_impl, file, proto) newXS_flags(name, c_impl, file, proto, 0)
#else
#define newXSproto_portable(name, c_impl, file, proto) (PL_Sv=(SV*)newXS(name, c_impl, file), sv_setpv(PL_Sv, proto), (CV*)PL_Sv)
#endif /* !defined(newXS_flags) */

#if PERL_VERSION_LE(5, 21, 5)
#  define newXS_deffile(a,b) Perl_newXS(aTHX_ a,b,file)
#else
#  define newXS_deffile(a,b) Perl_newXS_deffile(aTHX_ a,b)
#endif

/* simple backcompat versions of the TARGx() macros with no optimisation */
#ifndef TARGi
#  define TARGi(iv, do_taint) sv_setiv_mg(TARG, iv)
#  define TARGu(uv, do_taint) sv_setuv_mg(TARG, uv)
#  define TARGn(nv, do_taint) sv_setnv_mg(TARG, nv)
#endif

#line 434 "CairoGObject.c"
#ifdef __cplusplus
extern "C" {
#endif
XS_EXTERNAL(boot_Cairo__GObject); /* prototype to pass -Wmissing-prototypes */
XS_EXTERNAL(boot_Cairo__GObject)
{
#if PERL_VERSION_LE(5, 21, 5)
    dVAR; dXSARGS;
#else
    dVAR; dXSBOOTARGSXSAPIVERCHK;
#endif

    PERL_UNUSED_VAR(cv); /* -W */
    PERL_UNUSED_VAR(items); /* -W */
#if PERL_VERSION_LE(5, 21, 5)
    XS_VERSION_BOOTCHECK;
#  ifdef XS_APIVERSION_BOOTCHECK
    XS_APIVERSION_BOOTCHECK;
#  endif
#endif


    /* Initialisation Section */

#line 276 "CairoGObject.xs"
{
	gperl_register_boxed (CAIRO_GOBJECT_TYPE_CONTEXT, "Cairo::Context",
	                      &context_wrapper_class);
	/* CAIRO_GOBJECT_TYPE_DEVICE not wrapped yet */
	gperl_register_boxed (CAIRO_GOBJECT_TYPE_PATTERN, "Cairo::Pattern",
	                      &pattern_wrapper_class);
	gperl_register_boxed (CAIRO_GOBJECT_TYPE_SURFACE, "Cairo::Surface",
	                      &surface_wrapper_class);
	gperl_register_boxed (CAIRO_GOBJECT_TYPE_RECTANGLE, "Cairo::Rectangle",
	                      &rectangle_wrapper_class);
	gperl_register_boxed (CAIRO_GOBJECT_TYPE_RECTANGLE_INT, "Cairo::RectangleInt",
	                      &rectangle_int_wrapper_class);
	gperl_register_boxed (CAIRO_GOBJECT_TYPE_REGION, "Cairo::Region",
	                      &region_wrapper_class);
	gperl_register_boxed (CAIRO_GOBJECT_TYPE_SCALED_FONT, "Cairo::ScaledFont",
	                      &scaled_font_wrapper_class);
	gperl_register_boxed (CAIRO_GOBJECT_TYPE_FONT_FACE, "Cairo::FontFace",
	                      &font_face_wrapper_class);
	gperl_register_boxed (CAIRO_GOBJECT_TYPE_FONT_OPTIONS, "Cairo::FontOptions",
	                      &font_options_wrapper_class);
}

#line 482 "CairoGObject.c"

    /* End of Initialisation Section */

#if PERL_VERSION_LE(5, 21, 5)
#  if PERL_VERSION_GE(5, 9, 0)
    if (PL_unitcheckav)
        call_list(PL_scopestack_ix, PL_unitcheckav);
#  endif
    XSRETURN_YES;
#else
    Perl_xs_boot_epilog(aTHX_ ax);
#endif
}

#ifdef __cplusplus
}
#endif
